@@ -5,34 +5,28 @@ no strict;
 @ISA = qw( Exporter Mail::Mbox::MessageParser );
 
 use strict;
-use warnings 'all';
+use Carp;
+use Mail::Mbox::MessageParser;
 
-our $VERSION = '1.00';
+use vars qw( $VERSION $DEBUG $CACHE %CACHE_OPTIONS $UPDATING_CACHE
+  $CACHE_MODIFIED );
 
-our $DEBUG = 0;
+$VERSION = '1.02';
+
+*DEBUG = \$Mail::Mbox::MessageParser::DEBUG;
+*dprint = \&Mail::Mbox::MessageParser::dprint;
+sub dprint;
 
 # The class-wide cache, which will be read and written when necessary. i.e.
 # read when an folder reader object is created which uses caching, and
 # written when a different cache is specified, or when the program exits, 
-our $CACHE = undef;
+$CACHE = undef;
 
-our %CACHE_OPTIONS = ();
+%CACHE_OPTIONS = ();
 
-our $UPDATING_CACHE = 0;
+$UPDATING_CACHE = 0;
 
-our $CACHE_MODIFIED = 0;
-
-#-------------------------------------------------------------------------------
-
-sub dprint
-{
-  return Mail::Mbox::MessageParser::dprint @_;
-}
-
-#-------------------------------------------------------------------------------
-
-die "Can not load " . __PACKAGE__ . ": Storable is not installed.\n"
-  unless _LOAD_STORABLE();
+$CACHE_MODIFIED = 0;
 
 #-------------------------------------------------------------------------------
 
@@ -54,6 +48,9 @@ sub _LOAD_STORABLE
 sub SETUP_CACHE
 {
   my $options = shift;
+
+  return "Can not load " . __PACKAGE__ . ": Storable is not installed.\n"
+    unless _LOAD_STORABLE();
   
   # Load Storable if we need to
   # See if the client is setting up a different cache
@@ -70,6 +67,8 @@ sub SETUP_CACHE
   _READ_CACHE() if -f $CACHE_OPTIONS{'file_name'};
 
   $CACHE_MODIFIED = 0;
+
+  return 'ok';
 }
 
 #-------------------------------------------------------------------------------
@@ -106,9 +105,15 @@ sub WRITE_CACHE
 
   dprint "Writing cache.";
 
+  # The mail box cache may contain sensitive information, so protect it
+  # from prying eyes.
+  my $oldmask = umask(077);
+
   # Serialize using Storable
   store($CACHE, $CACHE_OPTIONS{'file_name'});
 
+  umask($oldmask);
+
   $CACHE_MODIFIED = 0;
 }
 
@@ -118,7 +123,7 @@ sub WRITE_CACHE
 sub END
 {
   dprint "Exiting and writing cache if necessary"
-    if defined(&Mail::Mbox::MessageParser::dprint);
+    if defined(&dprint);
 
   WRITE_CACHE() if $CACHE_MODIFIED;
 }
@@ -133,7 +138,7 @@ sub new
   my $self  = {};
   bless ($self, $class);
 
-  die "Call SETUP_CACHE() before calling new()"
+  carp "Call SETUP_CACHE() before calling new()"
     unless defined $CACHE_OPTIONS{'file_name'};
 
   # We need to write the cache if the user fully parsed a previous file before
@@ -142,8 +147,8 @@ sub new
   # for 99% of the cases.
   WRITE_CACHE() if $CACHE_MODIFIED && !$UPDATING_CACHE;
 
-  die "Need file_name option" unless defined $options->{'file_name'};
-  die "Need file_handle option" unless defined $options->{'file_handle'};
+  carp "Need file_name option" unless defined $options->{'file_name'};
+  carp "Need file_handle option" unless defined $options->{'file_handle'};
 
   $self->{'file_handle'} = undef;
   $self->{'file_handle'} = $options->{'file_handle'}
@@ -193,6 +198,8 @@ sub reset
   # If we're in the middle of parsing this file, we need to reset the cache
   if ($UPDATING_CACHE)
   {
+    dprint "Resetting cache\n";
+
     my @stat = stat $self->{'file_name'};
 
     my $size = $stat[7];
@@ -214,8 +221,6 @@ sub _read_prologue
 {
   my $self = shift;
 
-  dprint "Reading mailbox prologue using cache";
-
   my $prologue_length = $CACHE->{$self->{'file_name'}}{'offsets'}[0];
 
   {
@@ -255,7 +260,9 @@ sub _validate_and_initialize_cache_entry
   my $time_stamp = $stat[9];
 
   if (exists $CACHE->{$self->{'file_name'}} &&
-      ($CACHE->{$self->{'file_name'}}{'size'} != $size ||
+      (!defined $CACHE->{$self->{'file_name'}}{'size'} ||
+       !defined $CACHE->{$self->{'file_name'}}{'time_stamp'} ||
+       $CACHE->{$self->{'file_name'}}{'size'} != $size ||
        $CACHE->{$self->{'file_name'}}{'time_stamp'} != $time_stamp))
   {
     dprint "Size or time stamp has changed for file " .
@@ -266,10 +273,14 @@ sub _validate_and_initialize_cache_entry
 
   if (exists $CACHE->{$self->{'file_name'}})
   {
+    dprint "Cache is valid";
+
     $UPDATING_CACHE = 0;
   }
   else
   {
+    dprint "Cache is invalid: \"$self->{'file_name'}\" has not been parsed";
+
     $CACHE->{$self->{'file_name'}}{'size'} = $size;
     $CACHE->{$self->{'file_name'}}{'time_stamp'} = $time_stamp;
     $CACHE->{$self->{'file_name'}}{'lengths'} = [];
@@ -341,6 +352,7 @@ Mail::Mbox::MessageParser::Cache - A cache-based mbox folder reader
       'file_handle' => $filehandle,
     } );
 
+  die $folder_reader unless ref $folder_reader;
   
   die "No cached information"
     if $Mail::Mbox::MessageParser::Cache::UPDATING_CACHE;
@@ -381,6 +393,7 @@ the Mail::Mbox::MessageParser documentation.
 Call this function once to set up the cache before creating any parsers. You
 must provide the location to the cache file. There is no default value.
 
+Returns an error string or 1 if there is no error.
 
 =item CLEAR_CACHE();
 
@@ -407,6 +420,8 @@ filename of the mailbox. This will be used as the cache key, so it's important
 that it fully defines the path to the mailbox. The I<file_handle> argument is
 the opened file handle to the mailbox. Both arguments are required.
 
+Returns a reference to a Mail::Mbox::MessageParser object, or a string
+describing the error.
 
 =head1 BUGS
 
